#pragma region Globale Variablen
// Header-Datei fr die Windowsfunktionen
#include <windows.h>
#include <d3d10.h>
#include <d3dx9.h>
#include <dxgi.h>

HWND hWnd;	// Das Alias zum Fenster

const UINT	Width = 1024, // Bildbreite
			Height = 768; // Bildhhe
const BOOL	WindowMode = false; // Fenstermodus?

ID3D10Device*			gDevice = NULL;			// Das Device
ID3D10RenderTargetView*	gRenderTarget = NULL;	// Das Render Target
IDXGISwapChain*			gSwapChain;				// Die Swap Chain

#pragma endregion

#pragma region Callbackfunktion: Auswerten der Fensternachrichten
// Unsere Callback-Funktion zum Auswerten der Fensternachrichten
LRESULT CALLBACK WndProc(HWND hWnd /* Das Fenster */, UINT Msg /* Die Nachricht */,
						 WPARAM wParam, LPARAM lParam) // Jeweils Eingabewerte
{
	// Wenn eine Beendigungsnachricht kommt beenden:
	if(Msg == WM_DESTROY) { PostQuitMessage(0); return 0; }
	
	// Wenn ESC gedrckt wurde, Anwendung auch verlassen:
	if(Msg == WM_KEYDOWN && wParam == VK_ESCAPE) { PostQuitMessage(0); return 0; }

	// Wenn Alt+E gedrckt wurde, Anwendung auch verlassen:
	if(Msg == WM_KEYDOWN && wParam == 'E') { PostQuitMessage(0); return 0; }

	// Andernfalls Nachricht einfach verarbeiten:
	return DefWindowProc(hWnd, Msg, wParam, lParam);
}
#pragma endregion

#pragma region Funktion: Ein Fenster erstellen
bool AssignApplicationAndCreateWindow(HINSTANCE hInstance)
{
	// WNDCLASS-Struktur fr die Applikationsbeschreibung fr Windows fllen
	WNDCLASS wc = {0}; // << Auch gleich mit 0 vorinitialisieren
	wc.hbrBackground = HBRUSH(GetStockObject(WHITE_BRUSH));	// Schwarzer Hintergrund
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);				// Normaler Cursor
	wc.hIcon = LoadIcon(NULL, IDC_ARROW);					// Anwendungsicon
	wc.hInstance = hInstance;							// Alias der Anwnedung bergeben
	wc.lpfnWndProc = WndProc;							// Unsere Callback-Funktion
	wc.lpszClassName = L"MyZFXApp";						// Identifikationsname
	wc.style = CS_HREDRAW|CS_VREDRAW;					// Anwendungsstil
	// Nun Applikation registrieren
	RegisterClass(&wc);	

	// Das Fenster erstellen
	if(NULL==(hWnd = CreateWindowEx(NULL /* Kein Stil */,
									wc.lpszClassName /* Der Identifikationsname */,
									L"Rock'n'DirectX: ZFX Tutorial Application" /* Fenstertitel */,
									WS_VISIBLE|WS_OVERLAPPEDWINDOW /* Aussehen des Fensters */,
									0, 0,
									GetSystemMetrics(0) /* X-Auflsung des Users */,
									GetSystemMetrics(1)-30 /* Y-Auflsung des Users */,
									NULL,		// Ist von keinem Fenster abhngig
									NULL,		// Kein Men
									hInstance,	// Anwendungsalias
									NULL)))		// Initialnachricht an das Elternfenster
		return false;

	return true;
}
#pragma endregion

#pragma region Funktion: Eine Nachricht in der Hauptschleife verarbeiten
bool RunWindow()
{
	MSG msg = {0};
	// Nachricht verarbeiten, falls vorhanden
	if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
	{
		// Wenn das Programm beendet werden soll, Hauptschleife verlassen
		if(msg.message == WM_QUIT) return false;

		// Nachricht verarbeiten
		DispatchMessage(&msg);
		TranslateMessage(&msg);
	}

	return true;
}
#pragma endregion

#pragma region Funktion: Initialisierung von DirectX 10
bool InitializeDirectX10()
{
	// Swap Chain Desc fllen (Konfiguration)
	DXGI_SWAP_CHAIN_DESC dsc = {0};
	dsc.BufferCount = 1;
	dsc.BufferDesc.Width = 1024;
	dsc.BufferDesc.Height = 768;
	dsc.BufferDesc.RefreshRate.Denominator = 1;		// 60Hz
	dsc.BufferDesc.RefreshRate.Numerator = 60;	
	dsc.Windowed = WindowMode;
	dsc.BufferDesc.Format = DXGI_FORMAT_R8G8B8A8_UNORM; // 32Bit + Alpha
	dsc.BufferUsage = DXGI_USAGE_RENDER_TARGET_OUTPUT;	// Als Render Target benutzen
	dsc.OutputWindow = hWnd;							// Unser Fenster bergeben
	dsc.SampleDesc.Count = 1;							// Qualitt des Samplings = 1
	dsc.SwapEffect = DXGI_SWAP_EFFECT_DISCARD;			// Pointertausch statt kopieren

	// Device erzeugen
	if(FAILED(D3D10CreateDeviceAndSwapChain(NULL /*Standardadapter*/,
											D3D10_DRIVER_TYPE_HARDWARE /* Hardware */,
											NULL /* Kein Softwaremodul */,
											0, // Keine Flags
											D3D10_SDK_VERSION, // SDK-Version benutzen
											&dsc, // Konfigurationsstrutktur
											&gSwapChain, // Swap Chain
											&gDevice))) // Device
		return false;

	// Render Target View erzeugen und dem Device zuordnen
	// Textur holen
	ID3D10Texture2D* BackBuffer = NULL;
	if(FAILED(gSwapChain->GetBuffer(0, __uuidof(ID3D10Texture2D), (void**)&BackBuffer)))
		return false;

	// Render Target View erstellen und zuordnen
	if(FAILED(gDevice->CreateRenderTargetView(BackBuffer, NULL, &gRenderTarget)))
		return false;
	BackBuffer->Release();
	gDevice->OMSetRenderTargets(1, &gRenderTarget, NULL);

	// View Port erzeugen und festlegen
	D3D10_VIEWPORT vp = {0};
	vp.TopLeftX = 0;	// Obere X-Ecke in px
	vp.TopLeftY = 0;	// Obere Y-Ecke in px
	vp.Width = 1024;	// Breite der Zeichenflche
	vp.Height = 768;	// Hhe der Zeichenflche
	vp.MinDepth = 0.0f;	// Minimaler Tiefenwert
	vp.MaxDepth = 1.0f;	// Maximaler Tiefenwert

	// Viewport festlegen
	gDevice->RSSetViewports(1, &vp);

	return true;
}

#pragma endregion

#pragma region Funktion: Aufrumen von DirectX 10
void ReleaseDirectX10()
{
	if(gRenderTarget) gRenderTarget->Release(); // Render Target releasen

	gSwapChain->SetFullscreenState(false, NULL); // Wichtig bei Vollbild
	if(gSwapChain) gSwapChain->Release(); // SwapChain releasen

	if(gDevice) gDevice->Release(); // Device releasen
}
#pragma endregion

#pragma region Initialisierungsfunktion
bool Initialize(HINSTANCE hInstance)
{
	if(!AssignApplicationAndCreateWindow(hInstance)) return false;
	if(!InitializeDirectX10()) return false;
	return true;
}
#pragma endregion

#pragma region Rendern
void Render()
{
	// Backbuffer mit Rot lschen:
	gDevice->ClearRenderTargetView(gRenderTarget, D3DXCOLOR(1.0f, 0.0f, 0.0f, 1.0f));
	gSwapChain->Present(1, 0);
}
#pragma endregion

#pragma region Release
void Release()
{
	ReleaseDirectX10();
}
#pragma endregion

#pragma region Unsere Startfunktion
int WINAPI WinMain(HINSTANCE hInstance /* Unser Alias zur Anwendung */,
				   HINSTANCE hPrevInstance /* Ehemaliger Alias */,
				   LPSTR lpCmdLine /* Kommandozeile, wenn vorhanden */,
				   int nShowCmd /* Art, wie das Fenster gestartet werden soll */)
{
	// Initialisieren
	if(!Initialize(hInstance)) return 0;

	// Die Hauptschleife
	while(RunWindow())
	{
		Render();
	}

	// Programm verlassen
	Release();

	return 0;
}

#pragma endregion